/*---------------------------------------------------------------------------------------*
                                Pic16f877A Eeprom library
Filename: eeprom.c
Controller: Pic16f877A
Oscillator: 20 MHz
Author: XploreLabz
website: www.xplorelabz.com

 ----------------------------------------------------------------------------------------*/

#include<htc.h>
#include"delay.h"


/*---------------------------------------------------------------------------------------
  void EEPROM_WriteByte(unsigned char eeprom_Address, unsigned char eeprom_Data)
 ----------------------------------------------------------------------------------------
 * I/P Arguments: char,char-->eeprom_address at which eeprom_data is to be written.
 * Return value	: none

 * description:This function is used to write the data at specified EEPROM_address..
           .
----------------------------------------------------------------------------------------*/
void EEPROM_WriteByte(unsigned char eeprom_Address, unsigned char eeprom_Data)
{
     while(RD || WR);      // check the WR&RD bit to see if a RD/WR is in progress
	 EEADR=eeprom_Address; // Write the address to EEADR.
                           // Make sure that the address is not larger than the memory size 
	 EEDATA=eeprom_Data;   // load the 8-bit data value to be written in the EEDATA register.
	 WREN=1;               // Set the WREN bit to enable eeprom operation.
     EECON2=0x55;          //Execute the special instruction sequence
	 EECON2=0xaa;          //Refer the datasheet for more info
	 WR=1;                 // Set the WR bit to trigger the eeprom write operation.
     delay_us(10);
     WREN=0;               // Disable eeprom operation.
}






/*-----------------------------------------------------------------------------------------
               unsigned char EEPROM_ReadByte(unsigned char eeprom_Address)
 ------------------------------------------------------------------------------------------
 * I/P Arguments: char-->eeprom_address from where eeprom_data is to be read.
 * Return value	: char-->data read from Eeprom.

 * description:
           This function is used to read the data from specified EEPROM_address.
-----------------------------------------------------------------------------------------*/
unsigned char EEPROM_ReadByte(unsigned char eeprom_Address)
{
    while(RD || WR);      // check the WR&RD bit to see if a RD/WR is in progress
	EEADR=eeprom_Address; // Write the address to EEADR.
                          // Make sure that the address is not larger than the memory size 
	RD = 1;               // Set the WR bit to trigger the eeprom read operation.
    delay_us(50);
    return(EEDATA);       // Return the data read form eeprom.
 }






/*---------------------------------------------------------------------------------------
void EEPROM_WriteNBytes(unsigned char EepromAddr, unsigned char *RamAddr, char NoOfBytes)
 ----------------------------------------------------------------------------------------
 * I/P Arguments: char,-->eeprom_address from where the N-bytes are to be written.
                  char*-->Pointer to the N-bytes of data to be written.
                  char --> Number of bytes to be written

 * Return value	: none

 * description:
            This function is used to write N-bytes of data at specified EEPROM_address.
            EEPROM_WriteByte() function is called to write a byte at atime.
            Source(RAM) and destination(EEPROM) address are incremented after each write
            NoOfBytes is Decemented each time a byte is written.
            Above Operation is carried out till all the bytes are written(NoOfBytes!=0)
---------------------------------------------------------------------------------------*/
void EEPROM_WriteNBytes(unsigned char EepromAddr, unsigned char *RamAddr, char NoOfBytes)
 {
  while(NoOfBytes !=  0)
   {
	EEPROM_WriteByte(EepromAddr,*RamAddr); //Write a byte from RAM to EEPROM
	    EepromAddr++;					   //Incerement the Eeprom Address
		RamAddr++;						   //Increment the RAM Address
		NoOfBytes--;					   //Decrement NoOfBytes after writing each Byte
	   }
 }



/*---------------------------------------------------------------------------------------
void EEPROM_ReadNBytes(unsigned char EepromAddr, unsigned char *RamAddr, char NoOfBytes)
 ----------------------------------------------------------------------------------------
 * I/P Arguments: char,-->eeprom_address from where the N-bytes is to be read.
                  char*-->Pointer into which the N-bytes of data is to be read.
                  char --> Number of bytes to be Read

 * Return value	: none

 * description:
            This function is used to Read N-bytes of data from specified EEPROM_address.
            EEPROM_ReadByte() func is called to read a byte at a time.
            Source(RAM) and destination(EEPROM) address are incremented each time.
            NoOfBytes is Decemented after a byte is read.
            Above Operation is carried out till all the bytes are read(NoOfBytes!=0)
---------------------------------------------------------------------------------------*/
void EEPROM_ReadNBytes(unsigned char EepromAddr, unsigned char *RamAddr, char NoOfBytes)
 {
   while(NoOfBytes !=  0)
	{
	 *RamAddr = EEPROM_ReadByte(EepromAddr);//Read a byte from EEPROM to RAM
	    EepromAddr++;						//Incerement the Eeprom Address
	 	RamAddr++;							//Increment the RAM Address
	 	NoOfBytes--;						//Decrement NoOfBytes after Reading each Byte

	 }
 }







/*---------------------------------------------------------------------------------------
 void EEPROM_WriteString(unsigned char eeprom_address, unsigned char * source_address)
 ----------------------------------------------------------------------------------------
 * I/P Arguments: char,-->eeprom_address where the String is to be written.
                  char*-->Pointer to String which has to be written.

 * Return value	: none

 * description:This function is used to Write a String at specified EEPROM_address.
               EEPROM_WriteByte() function is called to write a byte at a time.
               Source(RAM) and destination(EEPOM) address are incremented each time.
               Above Operation is carried out till Null char is identified.

   NOTE: Null char is also written into the eeprom.
---------------------------------------------------------------------------------------*/
void EEPROM_WriteString(unsigned char eeprom_address, unsigned char * source_address)
  {
    do
 	{
 	  EEPROM_WriteByte(eeprom_address,*source_address); //Write a byte from RAM to EEPROM
 	    source_address++;								//Incerement the RAM Address
 		eeprom_address++;								//Increment the Eeprom Address
 	  }while(*(source_address-1)!=0);
  }





/*---------------------------------------------------------------------------------------
void EEPROM_ReadString(unsigned char eeprom_address, unsigned char * destination_address)
 ----------------------------------------------------------------------------------------
 * I/P Arguments: char,-->eeprom_address from where the String is to be read.
                  char*-->Pointer into which the String is to be read.

 * Return value	: none

 * description:This function is used to Read a String from specified EEPROM_address.
               EEPROM_ReadByte() function is called to read a byte at a time.
               Source(EEPROM) and destination(RAM) address are incremented each time.
               Above Operation is carried out till Null char is identified.
---------------------------------------------------------------------------------------*/
void EEPROM_ReadString(unsigned char eeprom_address, unsigned char * destination_address)
  {
   char eeprom_data;

     do
 	 {
	  eeprom_data = EEPROM_ReadByte(eeprom_address); //Read a byte from EEPROM to RAM
 	    *destination_address = eeprom_data;			 //Copy the data into String Buffer
 	     destination_address++;						 //Incerement the RAM Address
 	 	 eeprom_address++;							 //Increment the Eeprom Address
 		}while(eeprom_data!=0);
  }





/*-----------------------------------------------------------------------------------------
                                   void EEPROM_Erase()
 ------------------------------------------------------------------------------------------
 * I/P Arguments: none

 * Return value	: none

 * description:This function is used to erase the entire Eeprom memory.
               Eeprom is filled with 0xFF to accomplish the Eeprom Erase.
               EEPROM_WriteByte() function is called to write a byte at a time.
               Whole memory(0-255) is traversed and filled with 0xFF
-----------------------------------------------------------------------------------------*/
  void EEPROM_Erase()
   {
	 unsigned char eeprom_address;

	 for(eeprom_address=0;eeprom_address<255;eeprom_address++)
	    {
		   EEPROM_WriteByte(eeprom_address,0xff); // Write Each memory location with OxFF
	      }
   }








