/*----------------------------------------------------------------------------------
                        PIC 4x4 Keypad Library
Filename: keypad.c
Controller: Pica6f877A
Oscillator: 20 MHz
Author: XploreLabz
website: www.xplorelabz.com

 ----------------------------------------------------------------------------------
 Note:
  1.Rows are connected to lower 4-bits of PORTC
  1.Cols are connected to higher 4-bits of PORTC
            ___________________
           |    |    |    |    |
           | 0  | 1  | 2  | 3  |--------- R0
           |____|____|____|____|
           |    |    |    |    |
		   | 4  | 5  | 6  | 7  |--------- R1
           |____|____|____|____|
           |    |    |    |    |
		   | 8  | 9  | A  | B  |--------- R2
		   |____|____|____|____|
		   |    |    |    |    |
		   | C  | D  | E  | F  |--------- R3
           |____|____|____|____|
             |    |    |    |
             |    |    |    |____________ C3
             |    |    |
             |    |    |_________________ C2
             |    |
             |    |______________________ C1
             |
             |___________________________ C0

----------------------------------------------------------------------------------*/



#include <htc.h>
#include "keypad.h"
#include "lcd.h"
#include "delay.h"

#define RowColDirection TRISC //Data Direction Configuration for keypad

#define ROW PORTC   //Lower four bits of P1 are used as ROWs
#define COL PORTC  //Higher four bits of P1 are used as COLs



/*---------------------------------------------------------------------------------
                   void KEYPAD_Init()
 ----------------------------------------------------------------------------------
 * I/P Arguments:none
 * Return value : none

 * description  : This function the rows and colums for keypad scan
        1.ROW lines are configured as Output.
        2.Column Lines are configured as Input.
-----------------------------------------------------------------------------------*/
void KEYPAD_Init()
{
      TRISD=0x00;
       RowColDirection=0x0f;     // Configure Row lines as O/P and Column lines as I/P 
}

void LCD_DisplayAscii(unsigned char key)
{
  unsigned char temp;
  temp =(key>> 4) & 0x0f;
 if(temp >9)
  temp+=0x07;
temp+=0x30;
LCD_DataWrite(temp);

temp =key & 0x0f;
if(temp >9)
  temp+=0x07;
temp+=0x30;
LCD_DataWrite(temp);


}

/*---------------------------------------------------------------------------------
                   void KEYPAD_WaitForKeyRelease()
 ----------------------------------------------------------------------------------
 * I/P Arguments:none

 * Return value	: none

 * description  : This function waits till the previous key is released.
        1.All the ROW lines are pulled high.
        2.Column Lines are read to check the key press.
        3.If all the Keys are released then Column lines will be low(0x00)
-----------------------------------------------------------------------------------*/
void KEYPAD_WaitForKeyRelease()
{
   unsigned char key=0;
    do
	  {
        ROW=0x00;           // Pull the ROW lines to high and Column lines low.
	    key=COL & 0x0f;     // Read the Columns, to check the key press
          PORTD=key;
       }while(key!=0x0F);   // Wait till the Key is released,
                            // If no Key is pressed, Column lines will be low(0x00)
}





/*---------------------------------------------------------------------------------
                   void KEYPAD_WaitForKeyPress()
 ----------------------------------------------------------------------------------
 * I/P Arguments:none

 * Return value	: none

 * description  : This function waits till a new key is pressed.
        1.All the ROW lines are pulled high.
        2.Column Lines are read to check the key press.
        3.If any Key is pressed then corresponding Column Line goes high.

        4.Wait for Some time and perform the above operation to ensure
          the True Key Press before decoding the KEY.
-----------------------------------------------------------------------------------*/
void KEYPAD_WaitForKeyPress()
{
  unsigned char key;
  do
   {
	 do
      {   PORTD=key;
	     ROW=0x00;   		  // Pull the ROW lines to high and Column lines low.
	    key=COL & 0x0F;	  // Read the Columns, to check the key press
	   }while(key==0x0F); // Wait till the Key is pressed,
						  // if a Key is pressed the corresponding Column line go high

	   delay_ms(1);		  // Wait for some time(debounce Time);

	    ROW=0x00;   		  // After debounce time, perform the above operation
	   key=COL & 0x0F;	  // to ensure the Key press.
           PORTD=key; 
	}while(key==0x0F);

 }





/*---------------------------------------------------------------------------------
                   unsigned char KEYPAD_ScanKey()
 ----------------------------------------------------------------------------------
 * I/P Arguments:none

 * Return value	: char--> Scancode of the Key Pressed

 * description  : This function scans all the rows to decode the key pressed.
        1.Each time a ROW line is pulled high to detect the KEY.
        2.Column Lines are read to check the key press.
        3.If any Key is pressed then corresponding Column Line goes high.

        4.Return the ScanCode(Combination of ROW & COL) for decoding the key.
-----------------------------------------------------------------------------------*/
unsigned char KEYPAD_ScanKey()
 {

	 unsigned char ScanKey = 0xE0,i, key;

   for(i=0;i<0x04;i++)            // Scan All the 4-Rows for key press
    {
      ROW=ScanKey;                // Select 1-Row at a time for Scanning the Key
      key=COL & 0x0F;             // Read the Column, for key press
        PORTD=key;
      if(key!=0x0F)               // If the KEY press is detected for the selected
          break;                   // ROW then stop Scanning,

     ScanKey=(ScanKey<<1)| 0x10; // Rotate the ScanKey to SCAN the remaining Rows
    }


    key = i | (ScanKey & 0xf0);         // Retun the row and COL status to decode the key
    return(key);
}







/*---------------------------------------------------------------------------------
                   unsigned char KEYPAD_GetKey()
 ----------------------------------------------------------------------------------
 * I/P Arguments:none

 * Return value	: char--> ASCII value of the Key Pressed

 * description: This function waits till a key is pressed and retuns its ASCII Value

        1.Wait till the previous key is released..
        2.Wait for the new key press.
        3.Scan all the rows one at atime for the pressed key.
        4.Decode the key pressed depending on ROW-COL combination and retuns its
          ASCII value.
-----------------------------------------------------------------------------------*/
unsigned char KEYPAD_GetKey()
{
	unsigned char key;
      LCD_DataWrite('b'); 
      KEYPAD_WaitForKeyRelease();    // Wait for the previous key release
      delay_ms(1);
      LCD_DataWrite('r');
      KEYPAD_WaitForKeyPress();      // Wait for the new key press
     LCD_DataWrite('s');
      key = KEYPAD_ScanKey();        // Scan for the key pressed.
   //  LCD_DisplayAscii(key);
     LCD_DataWrite(' ');
  LCD_CmdWrite(0xcd);
    LCD_DisplayAscii(key);
   switch(key)                       // Decode the key
     {
         case 0x11: key='7'; break;
         case 0x12: key='8'; break;
         case 0x14: key='9'; break;
         case 0x18: key='%'; break;
  		 case 0x21: key='4'; break;
		 case 0x22: key='5'; break;
	 	 case 0x24: key='6'; break;
	 	 case 0x28: key='*'; break;
	  	 case 0x41: key='1'; break;
		 case 0x42: key='2'; break;
		 case 0x44: key='3'; break;
	 	 case 0x48: key='-'; break;
		 case 0xFE: key='C'; break;
		 case 0xFD: key='D'; break;
		 case 0xFB: key='E'; break;
		 case 0x87: key='+'; break;
         default: key='z';
      }
   return(key);                      // Return the key
}
